@@ -14,8 +14,8 @@ from tinycoder.edit_parser import EditParser
 from tinycoder.file_manager import FileManager
 from tinycoder.git_manager import GitManager
 from tinycoder.input_preprocessor import InputPreprocessor
-from tinycoder.llms.base import LLMClient
 from tinycoder.llm_response_processor import LLMResponseProcessor
+import zenllm as llm
 from tinycoder.prompt_builder import PromptBuilder
 from tinycoder.repo_map import RepoMap
 from tinycoder.rule_manager import RuleManager
@@ -51,7 +51,6 @@ class App:
     def __init__(
         self,
         logger: logging.Logger,
-        client: LLMClient,
         model: str,
         git_manager: GitManager,
         git_root: Optional[str],
@@ -70,7 +69,8 @@ class App:
         """Initializes the App with its dependencies."""
         self.state = AppState()
         self.logger = logger
-        self.client = client
+        
+
         self.model = model
         self.git_manager = git_manager
         self.git_root = git_root
@@ -86,6 +86,9 @@ class App:
         self.prompt_session = prompt_session
         self.style = style
         self.formatter = AppFormatter()
+        # Track provider/base_url explicitly for zenllm calls
+        self.current_provider: Optional[str] = None
+        self.current_base_url: Optional[str] = None
 
         # Initialize context manager
         self.context_manager = ContextManager(
@@ -99,10 +102,11 @@ class App:
 
         # Initialize LLM response processor
         self.llm_processor = LLMResponseProcessor(
-            client=self.client,
             model=self.model,
             style=self.style,
-            logger=self.logger
+            logger=self.logger,
+            provider=self.current_provider,
+            base_url=self.current_base_url,
         )
 
         # Initialize Docker automation
@@ -162,7 +166,7 @@ class App:
             for i, fname in enumerate(suggested_files):
                 self.logger.info(f"  {i+1}. {self.formatter.format_filename(fname)}")
 
-            confirm_prompt = f"{self.formatter.format_warning('Add files to context? (y/N, or list indices like \'1,3\'): ')}"
+            confirm_prompt = "Add files to context? (y/N, or list indices like '1,3'): "
             confirm = prompt_user_input(confirm_prompt).strip().lower()
             if not confirm: # User cancelled
                 self.logger.info(self.formatter.format_warning("\nFile addition cancelled by user."))
@@ -223,6 +227,8 @@ class App:
             add_repomap_exclusion_func=self.repo_map.add_user_exclusion,
             remove_repomap_exclusion_func=self.repo_map.remove_user_exclusion,
             get_repomap_exclusions_func=self.repo_map.get_user_exclusions,
+            get_model_func=self._get_current_model,
+            set_model_func=self._set_current_model,
         )
         self.logger.debug("CommandHandler initialized.")
 
@@ -240,6 +246,99 @@ class App:
         """Handle Docker automation after file modifications."""
         self.docker_automation.handle_modified_files(modified_files_rel, non_interactive)
 
+    def _get_current_model(self) -> str:
+        """Return the current LLM model identifier."""
+        return self.model
+
+    def _set_current_model(self, model_id: str, provider_key: Optional[str] = None, base_url: Optional[str] = None) -> None:
+        """Update the current model for this session and persist preference."""
+        old_model = self.model
+
+        # Canonicalize runtime model based on provider/base_url
+        runtime_model = model_id
+        provider_norm = (provider_key or "").lower() if provider_key else None
+
+        if not base_url and provider_norm:
+            if provider_norm == "groq" and not runtime_model.startswith("groq-"):
+                runtime_model = f"groq-{runtime_model}"
+            elif provider_norm == "together" and not runtime_model.startswith("together-"):
+                runtime_model = f"together-{runtime_model}"
+            elif provider_norm in ("anthropic", "claude") and not runtime_model.startswith("claude-"):
+                runtime_model = f"claude-{runtime_model}"
+            elif provider_norm == "gemini" and not runtime_model.startswith("gemini-"):
+                runtime_model = f"gemini-{runtime_model}"
+            elif provider_norm == "deepseek" and not runtime_model.startswith("deepseek-"):
+                runtime_model = f"deepseek-{runtime_model}"
+            elif provider_norm == "xai":
+                # Accept native X.ai ids that start with grok-; otherwise ensure xai- prefix
+                if not (runtime_model.startswith("grok-") or runtime_model.startswith("xai-")):
+                    runtime_model = f"xai-{runtime_model}"
+
+        # Track provider/base_url explicitly for future calls
+        self.current_provider = ("anthropic" if provider_norm == "claude" else provider_norm) if provider_norm else None
+        self.current_base_url = base_url
+
+        # Apply runtime change
+        self.model = runtime_model
+        if hasattr(self, "llm_processor"):
+            self.llm_processor.model = runtime_model
+            self.llm_processor.provider = self.current_provider
+            self.llm_processor.base_url = self.current_base_url
+
+        # Log and add a tool message to history
+        try:
+            if old_model and old_model != runtime_model:
+                self.logger.info(f"Switched model: {old_model} -> {runtime_model}")
+            else:
+                self.logger.info(f"Switched model to: {runtime_model}")
+            label = f" ({self.current_provider})" if self.current_provider else ""
+            self.history_manager.save_message_to_file_only("tool", f"Switched model to: {runtime_model}{label}")
+        except Exception:
+            pass
+
+        # Persist preference
+        try:
+            from tinycoder.preferences import load_user_preferences, save_user_preferences
+            prefs = load_user_preferences()
+
+            # Normalize provider key for persistence (map 'claude' to 'anthropic')
+            provider_to_save = None
+            if provider_norm:
+                provider_to_save = "anthropic" if provider_norm == "claude" else provider_norm
+
+            if base_url and not provider_to_save:
+                # For custom base_url without a known provider, store raw model string
+                prefs["model"] = model_id
+            elif provider_to_save or base_url:
+                # Derive unprefixed name for storage when applicable
+                name_to_save = model_id
+                if provider_to_save == "anthropic" and name_to_save.startswith("claude-"):
+                    name_to_save = name_to_save[len("claude-"):]
+                elif provider_to_save == "gemini" and name_to_save.startswith("gemini-"):
+                    name_to_save = name_to_save[len("gemini-"):]
+                elif provider_to_save == "deepseek" and name_to_save.startswith("deepseek-"):
+                    name_to_save = name_to_save[len("deepseek-"):]
+                elif provider_to_save == "together" and name_to_save.startswith("together-"):
+                    name_to_save = name_to_save[len("together-"):]
+                elif provider_to_save == "groq" and name_to_save.startswith("groq-"):
+                    name_to_save = name_to_save[len("groq-"):]
+                elif provider_to_save == "xai" and name_to_save.startswith("xai-"):
+                    name_to_save = name_to_save[len("xai-"):]
+
+                prefs["model"] = {
+                    "provider": provider_to_save,
+                    "name": name_to_save,
+                    "full_name": runtime_model,
+                    "base_url": base_url,
+                }
+            else:
+                prefs["model"] = runtime_model
+
+            save_user_preferences(prefs)
+        except Exception as e:
+            # Non-fatal; just log at debug level
+            self.logger.debug(f"Could not persist model preference: {e}")
+
 
     def _add_initial_files(self, files: List[str]) -> None:
         """Adds initial files specified via command line arguments."""
@@ -280,8 +379,27 @@ class App:
         else:
             files_str = "  Files: none"
         
-        # Create multi-line toolbar with files on top line and tokens below
-        toolbar_text = f"{files_str}\n{self.formatter.format_bottom_toolbar(breakdown)}"
+        # Determine provider/model display for toolbar
+        provider_display = self.current_provider
+        if not provider_display and self.model:
+            ml = self.model.lower()
+            if ml.startswith("claude-"):
+                provider_display = "anthropic"
+            elif any(ml.startswith(p + "-") for p in ("groq", "together", "gemini", "deepseek", "xai")):
+                provider_display = ml.split("-", 1)[0]
+            elif ml.startswith("grok-"):
+                provider_display = "xai"
+            elif ":" in self.model and "/" not in self.model and " " not in self.model:
+                provider_display = "ollama"
+        provider_display = provider_display or "auto"
+        provider_model_str = self.formatter.format_provider_model_line(provider_display, self.model)
+        
+        # Create multi-line toolbar with files, tokens, and provider/model info
+        toolbar_text = (
+            f"{files_str}\n"
+            f"{self.formatter.format_bottom_toolbar(breakdown)}\n"
+            f"{provider_model_str}"
+        )
         return toolbar_text
 
     def _update_and_cache_token_breakdown(self) -> None:
@@ -443,23 +561,43 @@ class App:
         
         valid_files_to_potentially_add = []
         non_existent_files_requested = []
+        out_of_scope_files_requested = []
+        not_regular_files_requested = []
         already_in_context_files = []
 
         for fname_rel in requested_files_from_llm:
             abs_path = self.file_manager.get_abs_path(fname_rel)
-            if abs_path and abs_path.exists():
-                if fname_rel not in self.file_manager.get_files():
-                     valid_files_to_potentially_add.append(fname_rel)
-                else:
-                    already_in_context_files.append(fname_rel)
-            else:
+            if abs_path is None:
+                out_of_scope_files_requested.append(fname_rel)
+                continue
+            if not abs_path.exists():
                 non_existent_files_requested.append(fname_rel)
+                continue
+            if not abs_path.is_file():
+                not_regular_files_requested.append(fname_rel)
+                continue
+            if fname_rel not in self.file_manager.get_files():
+                valid_files_to_potentially_add.append(fname_rel)
+            else:
+                already_in_context_files.append(fname_rel)
+
+        if out_of_scope_files_requested:
+            formatted_out_of_scope = [self.formatter.format_error(fname) for fname in out_of_scope_files_requested]
+            self.logger.warning(
+                f"LLM requested path(s) outside the project root (rejected): {', '.join(formatted_out_of_scope)}"
+            )
 
         if non_existent_files_requested:
             formatted_non_existent = [self.formatter.format_error(fname) for fname in non_existent_files_requested]
             self.logger.warning(
                 f"LLM requested non-existent files: {', '.join(formatted_non_existent)}"
             )
+
+        if not_regular_files_requested:
+            formatted_not_regular = [self.formatter.format_error(fname) for fname in not_regular_files_requested]
+            self.logger.warning(
+                f"LLM requested non-regular files (directories or special files): {', '.join(formatted_not_regular)}"
+            )
         
         if already_in_context_files:
             formatted_already_in_context = [self.formatter.format_info(fname) for fname in already_in_context_files]
@@ -486,7 +624,7 @@ class App:
         for i, fname in enumerate(valid_files_to_potentially_add):
             self.logger.info(f"  {i+1}. {self.formatter.format_filename(fname)}")
         
-        confirm_prompt = f"{self.formatter.format_warning('Add these files to context? (y/N, or list indices like \'1,3\'): ')}"
+        confirm_prompt = "Add these files to context? (y/N, or list indices like '1,3'): "
         confirm = (await self._prompt_for_confirmation(confirm_prompt)).strip().lower()
 
         if not confirm: # Handles cancellation from prompt_user_input
@@ -600,7 +738,7 @@ class App:
                     combined_errors = "\n".join(error_messages)
                     self.logger.error(combined_errors)
 
-                    fix_lint = await self._prompt_for_confirmation(f"{self.formatter.format_warning('Attempt to fix lint errors? (y/N): ')}")
+                    fix_lint = await self._prompt_for_confirmation("Attempt to fix lint errors? (y/N): ")
                     if fix_lint.lower() == "y":
                         self.state.reflected_message = combined_errors
             
@@ -617,15 +755,26 @@ class App:
 
         history_for_files = [{"role": "user", "content": instruction}]
         try:
-            response_content, error_message = self.client.generate_content(
-                system_prompt=system_prompt, history=history_for_files
+            # Prepare model/provider/base_url for zenllm call
+            call_model = self.model
+            p = (self.current_provider or "").lower() if getattr(self, "current_provider", None) else ""
+            # Do NOT strip for providers with native prefixed IDs (anthropic/claude-*, gemini/gemini-*, deepseek/deepseek-*)
+            if p in ("groq", "together") and call_model.startswith(f"{p}-"):
+                call_model = call_model[len(p) + 1:]
+            elif p == "xai" and call_model.startswith("xai-"):
+                call_model = call_model[len("xai-"):]
+            resp = llm.chat(
+                [("system", system_prompt), ("user", instruction)],
+                model=call_model,
+                provider=self.current_provider,
+                base_url=getattr(self, "current_base_url", None),
             )
+            response_content = resp.text or ""
         except KeyboardInterrupt:
             self.logger.info("\nLLM file suggestion cancelled.")
             return None  # Return None on cancellation to indicate user wants to exit
-
-        if error_message:
-            self.logger.error(f"Error asking LLM for files: {error_message}")
+        except Exception as e:
+            self.logger.error(f"Error asking LLM for files: {e}")
             return []
         if not response_content:
             self.logger.warning("LLM did not suggest any files.")
@@ -640,26 +789,54 @@ class App:
         # Basic filtering: remove backticks or quotes if LLM included them
         potential_files = [f.strip("`\"' ") for f in potential_files]
 
-        # Filter out files that don't exist in the repository
-        existing_files = []
+        # Filter out files that don't exist or are out of project scope
+        in_scope_existing_files = []
+        out_of_scope_files = []
+        non_existing_files = []
+        not_regular_files = []
+        history_rel = ".tinycoder.chat.history.md"
+
         for fname in potential_files:
+            if fname == history_rel:
+                self.logger.debug(f"Excluding internal history file from suggestions: {history_rel}")
+                continue
             abs_path = self.file_manager.get_abs_path(fname)
-            if abs_path and abs_path.exists():
-                existing_files.append(fname)
-            else:
-                self.logger.warning(
-                    f"Ignoring non-existent file suggested by LLM: {self.formatter.format_error(fname)}"
-                )
-        
-        if existing_files:
-            colored_existing_files = [f"{self.formatter.format_filename(f)}" for f in existing_files]
+            if abs_path is None:
+                out_of_scope_files.append(fname)
+                continue
+            if not abs_path.exists():
+                non_existing_files.append(fname)
+                continue
+            if not abs_path.is_file():
+                not_regular_files.append(fname)
+                continue
+            in_scope_existing_files.append(fname)
+
+        if out_of_scope_files:
+            formatted = [self.formatter.format_error(f) for f in out_of_scope_files]
+            self.logger.warning(
+                f"Ignoring path(s) outside the project root: {', '.join(formatted)}"
+            )
+        if non_existing_files:
+            formatted = [self.formatter.format_error(f) for f in non_existing_files]
+            self.logger.warning(
+                f"Ignoring non-existent file(s) suggested by LLM: {', '.join(formatted)}"
+            )
+        if not_regular_files:
+            formatted = [self.formatter.format_error(f) for f in not_regular_files]
+            self.logger.warning(
+                f"Ignoring non-regular path(s) (directories or special files): {', '.join(formatted)}"
+            )
+
+        if in_scope_existing_files:
+            colored_existing_files = [f"{self.formatter.format_filename(f)}" for f in in_scope_existing_files]
             self.logger.info(
                 f"LLM suggested files (after filtering): {', '.join(colored_existing_files)}",
             )
         else:
             self.logger.info("LLM suggested no existing files after filtering.")
             
-        return existing_files
+        return in_scope_existing_files
 
     def init_before_message(self):
         """Resets state before processing a new user message."""
@@ -785,6 +962,21 @@ class App:
         self._update_and_cache_token_breakdown()
 
         # Use logger for startup info, which has its own color formatting.
+        # Show provider aligned with the Model and /help lines
+        provider_display = self.current_provider
+        if not provider_display and self.model:
+            ml = self.model.lower()
+            if ml.startswith("claude-"):
+                provider_display = "anthropic"
+            elif any(ml.startswith(p + "-") for p in ("groq", "together", "gemini", "deepseek", "xai")):
+                provider_display = ml.split("-", 1)[0]
+            elif ml.startswith("grok-"):
+                provider_display = "xai"
+            elif ":" in self.model and "/" not in self.model and " " not in self.model:
+                provider_display = "ollama"
+        provider_display = provider_display or "auto"
+
+        self.logger.info(f"  Provider: {self.formatter.format_success(self.formatter.format_bold(provider_display))}")
         self.logger.info(f"  Model: {self.formatter.format_success(self.formatter.format_bold(self.model))}")
         self.logger.info("  Type /help for commands, or !<cmd> to run shell commands.\n")
 
