@@ -40,6 +40,8 @@ class CommandHandler:
         add_repomap_exclusion_func: Callable[[str], bool],
         remove_repomap_exclusion_func: Callable[[str], bool],
         get_repomap_exclusions_func: Callable[[], list[str]],
+        get_model_func: Callable[[], str],
+        set_model_func: Callable[[str, Optional[str], Optional[str]], None],
     ):
         """
         Initializes the CommandHandler.
@@ -65,6 +67,8 @@ class CommandHandler:
             add_repomap_exclusion_func: Function to add a path/pattern to repomap exclusions.
             remove_repomap_exclusion_func: Function to remove a path/pattern from repomap exclusions.
             get_repomap_exclusions_func: Function to get the list of current repomap exclusions.
+            get_model_func: Function to get the current model identifier.
+            set_model_func: Function to set the current model (model_id, provider_key, base_url).
         """
         self.file_manager = file_manager
         self.git_manager = git_manager
@@ -86,6 +90,8 @@ class CommandHandler:
         self.add_repomap_exclusion = add_repomap_exclusion_func
         self.remove_repomap_exclusion = remove_repomap_exclusion_func
         self.get_repomap_exclusions = get_repomap_exclusions_func
+        self.get_model = get_model_func
+        self.set_model = set_model_func
 
     # _run_tests method removed
 
@@ -298,6 +304,85 @@ class CommandHandler:
             run_coverage_summary(self.write_history_func, self.git_manager, self.logger)
             return True, None
 
+        elif command == "/stats":
+            # Analyze git log stats by keyword in commit subjects
+            if not self.git_manager.is_repo():
+                self.logger.error("Not in a git repository or Git is unavailable, cannot compute stats.")
+                return True, None
+
+            # Parse options: --keyword, --branch (support '=', space, and quoted values)
+            keyword = "tinycoder"
+            branch_opt = None
+            if args_str:
+                kw_match = re.search(r'--keyword(?:=|\s+)(?:"([^"]+)"|\'([^\']+)\'|(\S+))', args_str)
+                if kw_match:
+                    keyword = next(g for g in kw_match.groups() if g).strip()
+                br_match = re.search(r'--branch(?:=|\s+)(?:"([^"]+)"|\'([^\']+)\'|(\S+))', args_str)
+                if br_match:
+                    branch_opt = next(g for g in br_match.groups() if g).strip()
+
+            # Resolve branch to analyze
+            branch_to_use = branch_opt or self.git_manager.get_current_branch() or "HEAD"
+
+            self.logger.info(f"🔍 Analyzing repository history on branch '{branch_to_use}' for keyword: '{keyword}'...")
+
+            # Run git log with stats and a unique commit separator
+            ret, stdout, stderr = self.git_manager._run_git_command(
+                ["log", branch_to_use, "--stat", "--pretty=format:---COMMIT_SEPARATOR---%n%s"]
+            )
+            if ret != 0:
+                self.logger.error(f"Error executing git command: {stderr.strip()}")
+                return True, None
+
+            parts = stdout.split('---COMMIT_SEPARATOR---')
+            commits = parts[1:] if len(parts) > 1 else []
+
+            if not commits:
+                self.logger.info("No commits found in the repository.")
+                return True, None
+
+            keyword_lines_changed = 0
+            other_lines_changed = 0
+
+            for commit_block in commits:
+                lines = commit_block.strip().split('\n')
+                commit_message = lines[0] if lines else ""
+
+                commit_changes = 0
+                for line in lines:
+                    # Match summary lines like: "2 files changed, 10 insertions(+), 5 deletions(-)"
+                    if ("files changed" in line) or ("file changed" in line):
+                        ins_m = re.search(r'(\d+)\s+insertion', line)
+                        del_m = re.search(r'(\d+)\s+deletion', line)
+                        if ins_m:
+                            commit_changes += int(ins_m.group(1))
+                        if del_m:
+                            commit_changes += int(del_m.group(1))
+                        break
+
+                if keyword.lower() in commit_message.lower():
+                    keyword_lines_changed += commit_changes
+                else:
+                    other_lines_changed += commit_changes
+
+            total_lines_changed = keyword_lines_changed + other_lines_changed
+            percentage = (keyword_lines_changed / total_lines_changed) * 100 if total_lines_changed else 0.0
+
+            # Output summary
+            self.logger.info("\n--- Git Contribution Analysis ---")
+            self.logger.info(f"Total lines changed by '{keyword}': {keyword_lines_changed:,}")
+            self.logger.info(f"Total lines changed by others:   {other_lines_changed:,}")
+            self.logger.info("---------------------------------")
+            self.logger.info(f"Total lines changed in repository: {total_lines_changed:,}")
+            self.logger.info(f"Percentage of code by '{keyword}': {percentage:.2f}% ✅")
+
+            # Persist concise summary to chat history
+            self.write_history_func(
+                "tool",
+                f"Git stats: keyword='{keyword}' on '{branch_to_use}': {keyword_lines_changed} vs {other_lines_changed} (total {total_lines_changed}) -> {percentage:.2f}%"
+            )
+            return True, None
+
         elif command == "/docker":
             if not self.docker_manager or not self.docker_manager.is_available:
                 self.logger.error("Docker integration is not available or enabled.")
@@ -439,6 +524,109 @@ class CommandHandler:
 
             return True, None
 
+        elif command == "/model":
+            from tinycoder.ui.console_interface import prompt_user_input
+            import os
+            try:
+                import zenllm as llm
+            except Exception as e:
+                self.logger.error(f"Could not import zenllm: {e}")
+                return True, None
+
+            providers = [
+                ("openai",  "OpenAI (and compatible)",       ["OPENAI_API_KEY"]),
+                ("groq",    "Groq",                          ["GROQ_API_KEY"]),
+                ("anthropic","Anthropic (Claude)",           ["ANTHROPIC_API_KEY"]),
+                ("deepseek","DeepSeek",                      ["DEEPSEEK_API_KEY"]),
+                ("gemini",  "Google Gemini",                 ["GEMINI_API_KEY", "GOOGLE_API_KEY"]),
+                ("together","Together AI",                   ["TOGETHER_API_KEY"]),
+                ("xai",     "X.ai (Grok)",                   ["XAI_API_KEY"]),
+                ("custom",  "Custom base_url (OpenAI-compatible)", []),
+            ]
+
+            def _key_status(envs):
+                if not envs:
+                    return "n/a", True
+                has_any = any(os.getenv(v) for v in envs)
+                return ("OK" if has_any else "missing"), has_any
+
+            self.logger.info("Select a provider:")
+            for idx, (_k, label, envs) in enumerate(providers, start=1):
+                status_label, _ = _key_status(envs)
+                self.logger.info(f"  {idx}) {label}  [key: {status_label}]")
+            sel = prompt_user_input("Provider number (or Enter to cancel): ").strip()
+            if not sel:
+                return True, None
+            if not sel.isdigit() or not (1 <= int(sel) <= len(providers)):
+                self.logger.warning("Invalid provider selection.")
+                return True, None
+            pkey, plabel, penvs = providers[int(sel) - 1]
+
+            base_url = None
+            if pkey == "custom":
+                base_url = prompt_user_input("Enter base_url (e.g., http://localhost:11434/v1) or Enter to cancel: ").strip()
+                if not base_url:
+                    self.logger.info("Cancelled.")
+                    return True, None
+
+            # Warn if key missing
+            _, has_key = _key_status(penvs)
+            if penvs and not has_key:
+                cont = prompt_user_input("API key appears missing for this provider. Continue anyway? (y/N): ").strip().lower()
+                if cont != "y":
+                    return True, None
+
+            # List models
+            try:
+                if base_url:
+                    models = llm.list_models(base_url=base_url)
+                else:
+                    models = llm.list_models(provider=pkey)
+            except KeyboardInterrupt:
+                self.logger.info("Model listing cancelled.")
+                return True, None
+            except Exception as e:
+                self.logger.error(f"Failed to list models for provider '{pkey}': {e}")
+                return True, None
+
+            model_ids = sorted({getattr(m, "id", str(m)) for m in (models or [])})
+            if not model_ids:
+                self.logger.warning("No models returned by provider.")
+                return True, None
+
+            filt = prompt_user_input("Optional filter substring (Enter to skip): ").strip().lower()
+            if filt:
+                model_ids = [m for m in model_ids if filt in m.lower()]
+                if not model_ids:
+                    self.logger.warning("No models match the filter.")
+                    return True, None
+
+            max_show = 50
+            if len(model_ids) > max_show:
+                self.logger.info(f"Showing first {max_show} of {len(model_ids)} models.")
+                model_ids = model_ids[:max_show]
+
+            self.logger.info("Select a model:")
+            for idx, mid in enumerate(model_ids, start=1):
+                self.logger.info(f"  {idx}) {mid}")
+            msel = prompt_user_input("Model number (or Enter to cancel): ").strip()
+            if not msel:
+                return True, None
+            if not msel.isdigit() or not (1 <= int(msel) <= len(model_ids)):
+                self.logger.warning("Invalid model selection.")
+                return True, None
+
+            chosen_id = model_ids[int(msel) - 1]
+            try:
+                self.set_model(chosen_id, pkey if pkey != "custom" else None, base_url)
+            except Exception as e:
+                self.logger.error(f"Failed to set model: {e}")
+                return True, None
+
+            self.logger.info(f"Switched model to: {chosen_id} ({plabel})")
+            self.write_history_func("tool", f"Switched model to: {chosen_id} ({plabel})")
+            return True, None
+
         elif command == "/help":
             help_text = f"""Available commands:
   /add <file1> ["file 2"]...  Add file(s) to the chat context.
@@ -452,8 +640,10 @@ class CommandHandler:
   /undo                       Undo the last commit made by {self.app_name}.
   /ask                        Switch to ASK mode (answer questions, no edits).
   /code                       Switch to CODE mode (make edits).
+  /model                      Select provider and model interactively.
   /tests                      Run unit tests (runs in container if docker-compose.yml is present).
   /coverage                   Run coverage summary (unittest discovery; summary per file and total).
+  /stats [--keyword <word>] [--branch <name>] Show git contribution stats by keyword in commit subjects.
   /docker ps                  Show status of Docker containers.
   /docker logs <service>      Stream logs from a Docker container.
   /docker restart <service>   Restart a Docker container.
